@@ -24,8 +24,6 @@
  *
  * The terminal widget uses these functions to start commands with new controlling
  * pseudo-terminals and to resize pseudo-terminals.
- *
- * Since: 0.26
  */
 
 #include <config.h>
@@ -107,6 +105,7 @@ _vte_pty_reset_signal_handlers(void)
 	signal(SIGHUP,  SIG_DFL);
 #endif
 	signal(SIGINT,  SIG_DFL);
+	signal(SIGQUIT, SIG_DFL);
 	signal(SIGILL,  SIG_DFL);
 	signal(SIGABRT, SIG_DFL);
 	signal(SIGFPE,  SIG_DFL);
@@ -216,8 +215,6 @@ typedef struct {
 
 /**
  * VtePty:
- *
- * Since: 0.26
  */
 struct _VtePty {
         GObject parent_instance;
@@ -248,8 +245,6 @@ struct _VtePtyClass {
  * @pty: a #VtePty
  *
  * FIXMEchpe
- *
- * Since: 0.26
  */
 void
 vte_pty_child_setup (VtePty *pty)
@@ -559,8 +554,6 @@ __vte_pty_spawn (VtePty *pty,
  * If setting the window size failed, @error will be set to a #GIOError.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
- *
- * Since: 0.26
  */
 gboolean
 vte_pty_set_size(VtePty *pty,
@@ -615,8 +608,6 @@ vte_pty_set_size(VtePty *pty,
  * If getting the window size failed, @error will be set to a #GIOError.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
- *
- * Since: 0.26
  */
 gboolean
 vte_pty_get_size(VtePty *pty,
@@ -1334,8 +1325,6 @@ _vte_pty_open_with_helper(VtePty *pty,
  * discipline do multibyte backspace correctly.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
- *
- * Since: 0.26
  */
 gboolean
 vte_pty_set_utf8(VtePty *pty,
@@ -1350,7 +1339,7 @@ vte_pty_set_utf8(VtePty *pty,
         g_return_val_if_fail(VTE_IS_PTY(pty), FALSE);
 
         priv = pty->priv;
-        g_return_val_if_fail (priv->pty_fd > 0, FALSE);
+        g_return_val_if_fail (priv->pty_fd != -1, FALSE);
 
         if (tcgetattr(priv->pty_fd, &tio) == -1) {
                 int errsv = errno;
@@ -1387,8 +1376,6 @@ vte_pty_set_utf8(VtePty *pty,
  *
  * Cleans up the PTY, specifically any logging performed for the session.
  * The file descriptor to the PTY master remains open.
- *
- * Since: 0.26
  */
 void
 vte_pty_close (VtePty *pty)
@@ -1617,8 +1604,6 @@ vte_pty_class_init (VtePtyClass *klass)
          *
          * Controls how the session is recorded in lastlog, utmp, and wtmp,
          * and whether to use the GNOME PTY helper.
-         *
-         * Since: 0.26
          */
         g_object_class_install_property
                 (object_class,
@@ -1634,8 +1619,6 @@ vte_pty_class_init (VtePtyClass *klass)
          * VtePty:fd:
          *
          * The file descriptor of the PTY master.
-         *
-         * Since: 0.26
          */
         g_object_class_install_property
                 (object_class,
@@ -1656,8 +1639,6 @@ vte_pty_class_init (VtePtyClass *klass)
  * enumeration. See #GError for more information on error domains.
  *
  * Returns: the error domain for VTE PTY errors
- *
- * Since: 0.26
  */
 GQuark
 vte_pty_error_quark(void)
@@ -1705,8 +1686,6 @@ vte_pty_error_quark(void)
  * be no fallback to allocate a PTY using Unix98 PTY functions.
  *
  * Returns: (transfer full): a new #VtePty, or %NULL on error with @error filled in
- *
- * Since: 0.26
  */
 VtePty *
 vte_pty_new_sync (VtePtyFlags flags,
@@ -1734,8 +1713,6 @@ vte_pty_new_sync (VtePtyFlags flags,
  * and close it on finalize.
  *
  * Returns: (transfer full): a new #VtePty for @fd, or %NULL on error with @error filled in
- *
- * Since: 0.26
  */
 VtePty *
 vte_pty_new_foreign_sync (int fd,
