@@ -1,19 +1,19 @@
 /*
  * Copyright (C) 2002 Red Hat, Inc.
  *
- * This is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Library General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -36,15 +36,13 @@
 #define _vte_table_is_numeric(__c) \
 	(((__c) >= '0') && ((__c) <= '9'))
 #define _vte_table_is_numeric_list(__c) \
-	((((__c) >= '0') && ((__c) <= '9')) || (__c) == ';')
+	((((__c) >= '0') && ((__c) <= '9')) || (__c) == ';' || (__c) == ':')
 
 struct _vte_table {
 	struct _vte_matcher_impl impl;
-	GQuark resultq;
 	const char *result;
 	unsigned char *original;
 	gssize original_length;
-	int increment;
 	struct _vte_table *table_string;
 	struct _vte_table *table_number;
 	struct _vte_table *table_number_list;
@@ -189,12 +187,11 @@ _vte_table_free(struct _vte_table *table)
 	g_slice_free(struct _vte_table, table);
 }
 
-/* Add a string to the tree with the given increment value. */
 static void
 _vte_table_addi(struct _vte_table *table,
 		const unsigned char *original, gssize original_length,
 		const char *pattern, gssize length,
-		const char *result, GQuark quark, int inc)
+		const char *result)
 {
 	int i;
 	guint8 check;
@@ -214,31 +211,19 @@ _vte_table_addi(struct _vte_table *table,
 					  "`%s' and `%s' are indistinguishable.\n",
 					  table->result, result);
 
-		table->resultq = g_quark_from_string(result);
-		table->result = g_quark_to_string(table->resultq);
+		table->result = g_intern_string(result);
 		if (table->original != NULL) {
 			g_free(table->original);
 		}
 		table->original = g_memdup(original, original_length);
 		table->original_length = original_length;
-		table->increment = inc;
 		return;
 	}
 
 	/* All of the interesting arguments begin with '%'. */
 	if (pattern[0] == '%') {
-		/* Handle an increment. */
-		if (pattern[1] == 'i') {
-			_vte_table_addi(table, original, original_length,
-					pattern + 2, length - 2,
-					result, quark, inc + 1);
-			return;
-		}
-
 		/* Handle numeric parameters. */
-		if ((pattern[1] == 'd') ||
-		    (pattern[1] == '2') ||
-		    (pattern[1] == '3')) {
+		if (pattern[1] == 'd') {
 			/* Create a new subtable. */
 			if (table->table_number == NULL) {
 				subtable = _vte_table_new();
@@ -249,23 +234,20 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result, quark, inc);
+					result);
 			return;
 		}
 
 		/* Handle variable-length parameters. */
-		if ((pattern[1] == 'm') ||
-		    (pattern[1] == 'M')) {
+		if (pattern[1] == 'm') {
 			/* Build the "new" original using the initial portion
 			 * of the original string and what's left after this
 			 * specifier. */
-			if (pattern[1] == 'm') {
+			{
 				int initial;
 				GByteArray *b;
 
 				initial = original_length - length;
-				/* 0 args; we use 'M' to signal that zero is
-				 * not allowed.  */
 				b = g_byte_array_new();
 				g_byte_array_set_size(b, 0);
 				g_byte_array_append(b, original, initial);
@@ -273,7 +255,7 @@ _vte_table_addi(struct _vte_table *table,
 				_vte_table_addi(table, b->data, b->len,
 						(const char *)b->data + initial,
 						b->len - initial,
-						result, quark, inc);
+						result);
 				g_byte_array_free(b, TRUE);
 			}
 			/* Create a new subtable. */
@@ -286,7 +268,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result, quark, inc);
+					result);
 			return;
 		}
 
@@ -304,7 +286,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result, quark, inc);
+					result);
 			return;
 		}
 
@@ -325,7 +307,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result, quark, inc);
+					result);
 			return;
 		}
 
@@ -352,7 +334,7 @@ _vte_table_addi(struct _vte_table *table,
 				_vte_table_addi(subtable,
 						original, original_length,
 						pattern + 3, length - 3,
-						result, quark, inc);
+						result);
 			}
 			/* Also add a subtable for higher characters. */
 			if (table->table == NULL) {
@@ -369,7 +351,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 3, length - 3,
-					result, quark, inc);
+					result);
 			return;
 		}
 	}
@@ -392,26 +374,26 @@ _vte_table_addi(struct _vte_table *table,
 	/* Add the rest of the string to the subtable. */
 	_vte_table_addi(subtable, original, original_length,
 			pattern + 1, length - 1,
-			result, quark, inc);
+			result);
 }
 
 /* Add a string to the matching tree. */
 void
 _vte_table_add(struct _vte_table *table,
 	       const char *pattern, gssize length,
-	       const char *result, GQuark quark)
+	       const char *result)
 {
 	_vte_table_addi(table,
 			(const unsigned char *) pattern, length,
 			pattern, length,
-			result, quark, 0);
+			result);
 }
 
 /* Match a string in a subtree. */
 static const char *
 _vte_table_matchi(struct _vte_table *table,
 		  const gunichar *candidate, gssize length,
-		  const char **res, const gunichar **consumed, GQuark *quark,
+		  const char **res, const gunichar **consumed,
 		  unsigned char **original, gssize *original_length,
 		  struct _vte_table_arginfo_head *params)
 {
@@ -425,7 +407,6 @@ _vte_table_matchi(struct _vte_table *table,
 		*original = table->original;
 		*original_length = table->original_length;
 		*res = table->result;
-		*quark = table->resultq;
 		return table->result;
 	}
 
@@ -453,7 +434,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = i;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + i, length - i,
-					 res, consumed, quark,
+					 res, consumed,
 					 original, original_length, params);
 	}
 
@@ -463,7 +444,7 @@ _vte_table_matchi(struct _vte_table *table,
 		const char *local_result;
 
 		subtable = table->table_number_list;
-		/* Iterate over all numeric characters and ';'. */
+		/* Iterate over all numeric characters, ';' and ':'. */
 		for (i = 1; i < length; i++) {
 			if (!_vte_table_is_numeric_list(candidate[i])) {
 				break;
@@ -478,7 +459,7 @@ _vte_table_matchi(struct _vte_table *table,
 		/* Try and continue. */
 		local_result = _vte_table_matchi(subtable,
 					 candidate + i, length - i,
-					 res, consumed, quark,
+					 res, consumed,
 					 original, original_length,
 					 params);
 		if (local_result != NULL) {
@@ -506,7 +487,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = i;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + i, length - i,
-					 res, consumed, quark,
+					 res, consumed,
 					 original, original_length, params);
 	}
 
@@ -521,7 +502,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = 1;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + 1, length - 1,
-					 res, consumed, quark,
+					 res, consumed,
 					 original, original_length, params);
 	}
 
@@ -533,9 +514,11 @@ _vte_table_matchi(struct _vte_table *table,
 
 static void
 _vte_table_extract_numbers(GValueArray **array,
-			   struct _vte_table_arginfo *arginfo, long increment)
+			   struct _vte_table_arginfo *arginfo)
 {
 	GValue value = {0,};
+	GValue subvalue = {0,};
+	GValueArray *subarray = NULL;
 	gssize i;
 
         if (G_UNLIKELY (*array == NULL)) {
@@ -543,16 +526,31 @@ _vte_table_extract_numbers(GValueArray **array,
         }
 
 	g_value_init(&value, G_TYPE_LONG);
+	g_value_init(&subvalue, G_TYPE_VALUE_ARRAY);
 	i = 0;
 	do {
 		long total = 0;
-		for (; i < arginfo->length && arginfo->start[i] != ';'; i++) {
+		for (; i < arginfo->length && arginfo->start[i] != ';' && arginfo->start[i] != ':'; i++) {
 			gint v = g_unichar_digit_value (arginfo->start[i]);
 			total *= 10;
 			total += v == -1 ?  0 : v;
 		}
 		g_value_set_long(&value, CLAMP (total, 0, G_MAXUSHORT));
-		g_value_array_append(*array, &value);
+		if (i < arginfo->length && arginfo->start[i] == ':') {
+			if (subarray == NULL) {
+				subarray = g_value_array_new(2);
+			}
+			g_value_array_append(subarray, &value);
+		} else {
+			if (subarray == NULL) {
+				g_value_array_append(*array, &value);
+			} else {
+				g_value_array_append(subarray, &value);
+				g_value_set_boxed(&subvalue, subarray);
+				g_value_array_append(*array, &subvalue);
+				subarray = NULL;
+			}
+		}
 	} while (i++ < arginfo->length);
 	g_value_unset(&value);
 }
@@ -580,38 +578,20 @@ _vte_table_extract_string(GValueArray **array,
 	g_value_unset(&value);
 }
 
-static void
-_vte_table_extract_char(GValueArray **array,
-			struct _vte_table_arginfo *arginfo, long increment)
-{
-	GValue value = {0,};
-
-	g_value_init(&value, G_TYPE_LONG);
-	g_value_set_long(&value, *(arginfo->start) - increment);
-
-	if (G_UNLIKELY (*array == NULL)) {
-		*array = g_value_array_new(1);
-	}
-	g_value_array_append(*array, &value);
-	g_value_unset(&value);
-}
-
 /* Check if a string matches something in the tree. */
 const char *
 _vte_table_match(struct _vte_table *table,
 		 const gunichar *candidate, gssize length,
 		 const char **res, const gunichar **consumed,
-		 GQuark *quark, GValueArray **array)
+		 GValueArray **array)
 {
 	struct _vte_table *head;
 	const gunichar *dummy_consumed;
 	const char *dummy_res;
-	GQuark dummy_quark;
 	GValueArray *dummy_array;
 	const char *ret;
 	unsigned char *original, *p;
 	gssize original_length;
-	long increment = 0;
 	int i;
 	struct _vte_table_arginfo_head params;
 	struct _vte_table_arginfo *arginfo;
@@ -625,10 +605,6 @@ _vte_table_match(struct _vte_table *table,
 		consumed = &dummy_consumed;
 	}
 	*consumed = candidate;
-	if (G_UNLIKELY (quark == NULL)) {
-		quark = &dummy_quark;
-	}
-	*quark = 0;
 	if (G_UNLIKELY (array == NULL)) {
 		dummy_array = NULL;
 		array = &dummy_array;
@@ -667,7 +643,6 @@ _vte_table_match(struct _vte_table *table,
 		/* Got a literal match. */
 		*consumed = candidate + i;
 		*res = head->result;
-		*quark = head->resultq;
 		return *res;
 	}
 
@@ -675,7 +650,7 @@ _vte_table_match(struct _vte_table *table,
 
 	/* Check for a pattern match. */
 	ret = _vte_table_matchi(table, candidate, length,
-				res, consumed, quark,
+				res, consumed,
 				&original, &original_length,
 				&params);
 	*res = ret;
@@ -688,25 +663,14 @@ _vte_table_match(struct _vte_table *table,
 		do {
 			/* All of the interesting arguments begin with '%'. */
 			if (p[0] == '%') {
-				/* Handle an increment. */
-				if (p[1] == 'i') {
-					increment++;
-					p += 2;
-					continue;
-				}
 				/* Handle an escaped '%'. */
-				else if (p[1] == '%') {
+				if (p[1] == '%') {
 					p++;
 				}
 				/* Handle numeric parameters. */
-				else if ((p[1] == 'd') ||
-				    (p[1] == '2') ||
-				    (p[1] == '3') ||
-				    (p[1] == 'm') ||
-				    (p[1] == 'M')) {
+				else if ((p[1] == 'd') || (p[1] == 'm')) {
 					_vte_table_extract_numbers(array,
-								   arginfo,
-								   increment);
+								   arginfo);
 					p++;
 				}
 				/* Handle string parameters. */
@@ -714,16 +678,9 @@ _vte_table_match(struct _vte_table *table,
 					_vte_table_extract_string(array,
 								  arginfo);
 					p++;
-				}
-				/* Handle a parameter character. */
-				else if (p[1] == '+') {
-					_vte_table_extract_char(array,
-								arginfo,
-								p[2]);
-					p += 2;
 				} else {
 					_vte_debug_print (VTE_DEBUG_PARSE,
-							  "Invalid termcap sequence %s\n",
+                                                          "Invalid sequence %s\n",
 							  original);
 				}
 			} /* else Literal. */
@@ -747,8 +704,8 @@ _vte_table_printi(struct _vte_table *table, const char *lead, int *count)
 
 	/* Result? */
 	if (table->result != NULL) {
-		g_printerr("%s = `%s'(%d)\n", lead,
-			table->result, table->increment);
+		g_printerr("%s = `%s'\n", lead,
+			table->result);
 	}
 
 	/* Literal? */
@@ -859,6 +816,9 @@ print_array(GValueArray *array)
 				printf("\"%ls\"",
 				       (wchar_t*) g_value_get_pointer(value));
 			}
+			if (G_VALUE_HOLDS_BOXED(value)) {
+				print_array(g_value_get_boxed(value));
+			}
 		}
 		printf(")");
 		/* _vte_matcher_free_params_array(array); */
@@ -878,6 +838,12 @@ main(int argc, char **argv)
 		"]3;fook",
 		"[3;foo",
 		"[3;3m",
+		"[3;5;42m",
+		"[3;5:42m",
+		"[3:5:42m",
+		"[3;2;110;120;130m",
+		"[3;2:110:120:130m",
+		"[3:2:110:120:130m",
 		"[3;3mk",
 		"[3;3hk",
 		"[3;3h",
@@ -890,22 +856,18 @@ main(int argc, char **argv)
 	const gunichar *consumed;
 	char *tmp;
 	gunichar *candidate;
-	GQuark quark;
 	GValueArray *array;
 	g_type_init();
 	table = _vte_table_new();
-	_vte_table_add(table, "ABCDEFG", 7, "ABCDEFG", 0);
-	_vte_table_add(table, "ABCD", 4, "ABCD", 0);
-	_vte_table_add(table, "ABCDEFH", 7, "ABCDEFH", 0);
-	_vte_table_add(table, "ACDEFH", 6, "ACDEFH", 0);
-	_vte_table_add(table, "ACDEF%sJ", 8, "ACDEF%sJ", 0);
-	_vte_table_add(table, "ACDEF%i%mJ", 10, "ACDEF%dJ", 0);
-	_vte_table_add(table, "[%mh", 5, "move-cursor", 0);
-	_vte_table_add(table, "[%d;%d;%dm", 11, "set-graphic-rendition", 0);
-	_vte_table_add(table, "[%dm", 5, "set-graphic-rendition", 0);
-	_vte_table_add(table, "[m", 3, "set-graphic-rendition", 0);
-	_vte_table_add(table, "]3;%s", 7, "set-icon-title", 0);
-	_vte_table_add(table, "]4;%s", 7, "set-window-title", 0);
+	_vte_table_add(table, "ABCDEFG", 7, "ABCDEFG");
+	_vte_table_add(table, "ABCD", 4, "ABCD");
+	_vte_table_add(table, "ABCDEFH", 7, "ABCDEFH");
+	_vte_table_add(table, "ACDEFH", 6, "ACDEFH");
+	_vte_table_add(table, "ACDEF%sJ", 8, "ACDEF%sJ");
+	_vte_table_add(table, "[%mh", 5, "move-cursor");
+	_vte_table_add(table, "[%mm", 5, "character-attributes");
+	_vte_table_add(table, "]3;%s", 7, "set-icon-title");
+	_vte_table_add(table, "]4;%s", 7, "set-window-title");
 	printf("Table contents:\n");
 	_vte_table_print(table);
 	printf("\nTable matches:\n");
@@ -914,7 +876,7 @@ main(int argc, char **argv)
 		candidate = make_wide(p);
 		array = NULL;
 		_vte_table_match(table, candidate, strlen(p),
-				 &result, &consumed, &quark, &array);
+				 &result, &consumed, &array);
 		tmp = escape(p);
 		printf("`%s' => `%s'", tmp, (result ? result : "(NULL)"));
 		g_free(tmp);
